%  Model of two coils with pierced core 
%  The german BH curve requires a DC current away from zero in order 
%  to converge  
  flclear fem fem0 
  close all 

  %..Set switches.. 
  bhswtch = 'tabular'; 

  %..Number of turns and current values 
  %..The current in the AC coil is set to compute the inductance of the coil 
  Idc  = 5; 
  dcNt = 250; 
  Iac  = 5; 
  acNt = 200; 
  acNt = 200; 

  %..Create geometery..
  %....Core....
  crwin  = 12.5e-3;   
  crwout = 37.5e-3;
  crwleg = crwout - crwin;
  crhin  = 39e-3; 
  crhout = 63e-3; 
  rad    = 3e-3; 
  core   = fillet(rect2(-crwout,crwout,-crhout,crhout),'radii',rad) - ...
                  rect2(-crwin,crwin,-crhin,crhin); 
  %....Flux integration lines.... 
  fluxline1 = line1([-crwout,-crwin],[0,0]); 
  fluxline2 = line1([crwin,crwout],[0,0]); 
  %....AC coil....
  accoilw      = 10e-3; accoilh = 20e-3; 
  xspacer      = 2e-3; 
  accoilradin  = crwleg/2+xspacer; accoilradout = accoilradin+accoilw; 
  accoilxc     = crwin+crwleg/2;
  accoil_right = rect2(accoilradin,accoilradout,-accoilh/2,accoilh/2); 
  accoil       = accoil_right+move(accoil_right,-2*accoilradin-accoilw,0); 
  accoil       = move(accoil,accoilxc,0); 
  %....DC coil....
  dccoilh     = 10e-3; dccoilw = crwin/2;
  dccoilradin = (crhout-crhin)/2; dccoilradout = dccoilradin + dccoilh;   
  dccoilyc    = crhin+(crhout-crhin)/2; 
  dccoil_top  = rect2(-dccoilw/2,dccoilw/2,dccoilradin,dccoilradout); 
  dccoil      = dccoil_top + move(dccoil_top,0,-2*dccoilradin-dccoilh); 
  dccoil      = move(dccoil,0,dccoilyc);
  %....Air.... 
  xmax = 400e-3; ymax = 400e-3; 
  air = rect2(-xmax, xmax, -ymax, ymax);
  
  % Analyzed geometry
  clear c s 
  c.objs={fluxline1,fluxline2};
  c.name={'fluxline1','fluxline2'};
  c.tags={'g5','g6'};

  s.objs={air,accoil,dccoil,core};
  s.name={'air1','accoil','dccoil','core'};
  s.tags={'g1','g2','g3','g4'};

  fem.draw=struct('c',c,'s',s);
  fem.geom=geomcsg(fem);
  
  %..Plot geometry..
  if (0) 
    geomplot(fem,'edgelabels','on')
    return 
  end
  
  fem.const = {'om',              2*pi*50,...
               'Vmax',            28,...
               'Rpre',            4.0,...
               'Rpost',           0.1,...
               'Tfault',          45e-3,...
               'Tsmooth',         1e-3,...
               'lz',              25e-3,... 
               'flcrwleg',        crwleg,... 
               'crcross',         'lz*flcrwleg',... 
               'flacNt',          acNt,... 
               'accross',         accoilw*accoilh,...
               'flIac',           Iac,... 
               'fldcNt',          dcNt,... 
               'flIdc',           Idc,...
               'dccross',         dccoilw*dccoilh,...              
               'linmurfe',        '1000',... 
               'bha',             '2.12e-4',...
               'bhb',             '7.358',...
               'bhc',             '1.18e6',...
               'C1',              '.25',...
               'C2',              '.06'}; 
           
  % Functions
  clear fcns
  fcns{1}.type='inline';
  fcns{1}.name='rational(x,bha,bhb,bhc)';
  fcns{1}.expr='1/(bha + (1-bha)*x^bhb/(x^bhb+bhc))';
  fcns{1}.dexpr={'diff(1/(bha+(1-bha)*x^bhb/ (x^bhb+bhc)),x)',...
                 '0','0','0'};
  fcns{2}.type='inline';
  fcns{2}.name='hyperchord(x,C1,C2)';
  fcns{2}.expr='C2*x/(4*pi*1e-7*sinh(x/C1))';
  fcns{2}.dexpr={'diff(C2*x/(4*pi*1e-7*sinh(x/C1)),x)','0','0'};
  fcns{3}.type='interp';
  fcns{3}.name='bmurtabular';
  fcns{3}.method='linear';
  fcns{3}.extmethod='const';
  fcns{3}.filename='/mnt/dutita1/nw/domenico/software/bh_curve/mur_normb_data.txt';
  fem.functions = fcns;

  % Global expressions 
  fem.globalexpr = {'Vline',     'Vmax*sin(om*t)',...
                    'Vind1',     'flacNt*lz/accross*Eint1',...
                    'Vind2',     'flacNt*lz/accross*Eint2',...
                    'Bavrg1',    'Bint1/flcrwleg',...
                    'Bavrg2',    'Bint2/flcrwleg',...
                    'fluxt1',    'flacNt*lz*Btint1',...
                    'fluxt2',    'flacNt*lz*Btint2',...
                    'Rline',     'Rpre - (Rpre-Rpost)*flc2hs(t-Tfault,Tsmooth)'};

  % Initialize mesh
  fem.mesh=meshinit(fem);
%%  fem.mesh = meshrefine(fem); 
%%  fem.mesh = meshrefine(fem); 

   if (0)
     meshplot(fem)
     return 
   end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Application mode 0: nominal current computation 
  if (0) 
    clear appl
    appl.mode.class = 'PerpendicularCurrents';
    appl.module = 'ACDC';
    clear bnd
    bnd.type = {'A0','cont'};
    bnd.ind = {[1,2,3,36],[4:35,37:40]};
    appl.bnd = bnd;
    clear equ
    equ.init = {'1e-6*sqrt(x^2+y^2)',0,0,0,0,0};
    if (strcmp(bhswtch,'linear')==1)
      equ.mur = {'linmurfe',1,1,1,1,1};
    elseif (strcmp(bhswtch,'rational')==1)
      equ.mur = {'rational(normB,bha,2*bhb,bhc)',1,1,1,1,1};
    elseif (strcmp(bhswtch,'hyperchord')==1)
      equ.mur = {'hyperchord(normB,C1,C2)',1,1,1,1,1};
    elseif (strcmp(bhswtch,'tabular')==1)
      equ.mur = {'bmurtabular(normB)',1,1,1,1,1};
    else
      error('  error::undefined BH switch')
    end
	equ.Jez = {0,'-flacNt*flIac/accross','flacNt*flIac/accross',0,0,0};
    equ.ind = {[2,3],7,8,5,6,[1,4]};
    appl.equ = equ;
    fem.appl{1} = appl;
    fem.frame = {'ref'};
    fem.border = 1;
    clear units;
    units.basesystem = 'SI';
    fem.units = units;

    % ODE Settings
    clear ode
    ode.dim={'Itot'};
    ode.f={'0'};
    ode.init={'0'};
    ode.dinit={'0'};
    fem.ode=ode;

    % Multiphysics
    fem=multiphysics(fem);

    % Extend mesh
    fem.xmesh=meshextend(fem);

    % Solve problem
    fem.sol=femstatic(fem, ...
 		      'solcomp',{'Az'}, ...
		      'outcomp',{'Az'});

    Rpre  = 4; Rpost = .1; om = 2*pi*50; Vmax = 28; lz = 25e-3; 
    Wmtot = lz*postint(fem,'.5*Bx*Hx+.5*By*Hy','Edim',2,'Dl',[1:8]); 
    L     = 2*Wmtot /Iac^2;
    Xpre  = sqrt(Rpre^2  + om^2*L^2); Ipre = Vmax/Xpre; 
    Xpost = sqrt(Rpost^2 + om^2*L^2); Ipost = Vmax/Xpost; 
    fprintf('  The inductance of the coil   = %f H. \n', L)        
    fprintf('  Nominal current before fault = %f Amp. \n', Ipre)
    fprintf('  Nominal current after  fault = %f Amp. \n', Ipost)

    return 

  end %..Nominal current computation 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Application mode 1: generating initial guess for the DC coil flux 
  if (1) 
    clear appl
    appl.mode.class = 'PerpendicularCurrents';
    appl.module = 'ACDC';
    clear bnd
    bnd.type = {'A0','cont'};
    bnd.ind = {[1,2,3,36],[4:35,37:40]};
    appl.bnd = bnd;
    clear equ
    equ.mur = {'100',1,1,1,1,1};
    equ.Jez = {0,0,0,'fldcNt*flIdc/dccross',...
	       '-fldcNt*flIdc/dccross',0};
    equ.ind = {[2,3],7,8,5,6,[1,4]};
    appl.equ = equ;
    fem.appl{1} = appl;
    fem.frame = {'ref'};
    fem.border = 1;
    clear units;
    units.basesystem = 'SI';
    fem.units = units;

    % ODE Settings
    clear ode
    ode.dim={'Itot'};
    ode.f={'0'};
    ode.init={'0'};
    ode.dinit={'0'};
    fem.ode=ode;

    % Multiphysics
    fem=multiphysics(fem);

    % Extend mesh
    fem.xmesh=meshextend(fem);

    % Solve problem
    fprintf('  Generating initial guess for the DC coil flux ...\n')
    fem.sol=femstatic(fem, ...
 		      'solcomp',{'Az'}, ...
		      'outcomp',{'Az','Itot'});
    fprintf('  ... done! \n')
 
    fem0 = fem; 


  end %..Parametric solver in DC winding 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Application mode 2: generating initial guess for the AC coil flux 
  if (1) 
    clear appl
    appl.mode.class = 'PerpendicularCurrents';
    appl.module = 'ACDC';
    clear bnd
    bnd.type = {'A0','cont'};
    bnd.ind = {[1,2,3,36],[4:35,37:40]};
    appl.bnd = bnd;
    clear equ
    if (strcmp(bhswtch,'linear')==1)
      equ.mur = {'linmurfe',1,1,1,1,1};
    elseif (strcmp(bhswtch,'rational')==1)
      equ.mur = {'rational(normB,bha,2*bhb,bhc)',1,1,1,1,1};
    elseif (strcmp(bhswtch,'hyperchord')==1)
      equ.mur = {'hyperchord(normB,C1,C2)',1,1,1,1,1};
    elseif (strcmp(bhswtch,'tabular')==1)
      equ.mur = {'bmurtabular(normB)',1,1,1,1,1};
    else
      error('  error::undefined BH switch')
    end
    equ.Jez = {0,0,0,'fldcNt*flIdc/dccross',...
	       '-fldcNt*flIdc/dccross',0};
    equ.ind = {[2,3],7,8,5,6,[1,4]};
    appl.equ = equ;
    fem.appl{1} = appl;
    fem.frame = {'ref'};
    fem.border = 1;
    clear units;
    units.basesystem = 'SI';
    fem.units = units;

    % ODE Settings
    clear ode
    ode.dim={'Itot'};
    ode.f={'0'};
    ode.init={'0'};
    ode.dinit={'0'};
    fem.ode=ode;

    % Multiphysics
    fem=multiphysics(fem);

    % Extend mesh
    fem.xmesh=meshextend(fem);

    % Solve problem
    fprintf('  Generating initial guess for the DC coil flux ...\n')
    fem.sol=femstatic(fem, ...
                      'init', fem0.sol,... 
 		      'solcomp',{'Az'}, ...
		      'outcomp',{'Az','Itot'}, ...
		      'Pname','flIdc',... 
		      'Plist',[Idc],... 
                      'Maxiter',40,... 
		      'Ntol',1e-8);
    fprintf('  ... done! \n')
 
    fem0 = fem; 

  end %..Generatring initial guess for the AC flux 
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Application mode 3: transient simulation  
  % Saturation current in the DC winding
  % ODE solution in the AC winding 
 
  clear appl
  appl.mode.class = 'PerpendicularCurrents';
  appl.module = 'ACDC';
  clear prop
  prop.analysis='transient';
  appl.prop = prop;
  clear bnd
  bnd.type = {'A0','cont'};
  bnd.ind = {[1,2,3,36],[4:35,37:40]};
  appl.bnd = bnd;
  clear equ
  equ.init = {'1e-6*sqrt(x^2+y^2)',0,0,0,0,0};
  if (strcmp(bhswtch,'linear')==1)
    equ.mur = {'linmurfe',1,1,1,1,1};
  elseif (strcmp(bhswtch,'rational')==1)
    equ.mur = {'rational(normB,bha,2*bhb,bhc)',1,1,1,1,1};
  elseif (strcmp(bhswtch,'hyperchord')==1)
    equ.mur = {'hyperchord(normB,C1,C2)',1,1,1,1,1};
  elseif (strcmp(bhswtch,'tabular')==1)
    equ.mur = {'bmurtabular(normB)',1,1,1,1,1};
  else
    error('  error::undefined BH switch')
  end
  equ.Jez = {0,'flacNt*Itot/accross','-flacNt*Itot/accross',...
	       'fldcNt*flIdc/dccross','-fldcNt*flIdc/dccross',0};
  equ.ind = {[2,3],7,8,5,6,[1,4]};
  appl.equ = equ;
  fem.appl{1} = appl;
  fem.frame = {'ref'};
  fem.border = 1;
  clear units;
  units.basesystem = 'SI';
  fem.units = units;

  % Coupling variable elements
  clear elemcpl
  % Integration coupling variables
  clear elem
  elem.elem = 'elcplscalar';
  elem.g = {'1'};
  src = cell(1,1);
  clear bnd
  bnd.expr = {{{},'By',{}},{{},{},'By'},{{},'-Azxt',{}},{{},{},'-Azxt'},{},{}};
  bnd.ipoints = {{{},'4',{}},{{},{},'4'},{{},'4',{}},{{},'4','4'},{},{}};
  bnd.frame = {{{},'ref',{}},{{},{},'ref'},{{},'ref',{}},{{},'ref','ref'},{},{}};
  bnd.ind = {{'1','2','3','4','5','7','8','9','10','11','12','13','14', ...
    '15','16','17','18','19','20','21','22','23','24','25','26','27','28', ...
    '30','31','32','33','34','35','36','37','38','39','40'},{'6'},{'29'}};
  clear equ
  equ.expr = {{},{},{},{},{{},{},'-Ez',{}},{{},{},{},'Ez'}};
  equ.ipoints = {{},{},{},{},{{},{},'4',{}},{{},{},{},'4'}};
  equ.frame = {{},{},{},{},{{},'ref','ref','ref'},{{},{},'ref','ref'}};
  equ.ind = {{'1','3','4','5','6'},{'2'},{'7'},{'8'}};
  src{1} = {{},bnd,equ};
  elem.src = src;
  geomdim = cell(1,1);
  geomdim{1} = {};
  elem.geomdim = geomdim;
  elem.var = {'Bint1','Bint2','Btint1','Btint2','Eint1','Eint2'};
  elem.global = {'1','2','3','4','5','6'};
  elem.maxvars = {};
  elemcpl{1} = elem;
  fem.elemcpl = elemcpl;

  % ODE Settings
  clear ode
  ode.dim={'Itot'};
  ode.f={'Rline*Itot+Vind1+Vind2-Vline'};
  ode.init={'0'};
  ode.dinit={'0'};
  fem.ode=ode;

  % Multiphysics
  fem=multiphysics(fem);

  % Extend mesh
  fem.xmesh=meshextend(fem);

  % Solve problem
  fem.sol=femtime(fem, ...
                  'init', fem0.sol,... 
		  'solcomp',{'Az','Itot'}, ...
		  'outcomp',{'Az','Itot'}, ...
		  'atol',1e-6,...
		  'rtol',1e-6,... 
		  'tlist',[0:0.001:0.08], ...
		  'tout','tlist',...
                  'nlsolver','manual', ...
                  'ntolfact',0.01, ...
                  'maxiter',25, ...
                  'dtech','const', ...
                  'damp',1.0, ...
		  'jtech','minimal',...  
	          'linsolver','pardiso', ...
	          'errorchk','off');

  % Plot current 
  postglobalplot(fem,'Itot')
  postglobalplot(fem,{'Bavrg1','Bavrg2'})
  postglobalplot(fem,'Itot')
